hexbin <-
    function(x, y, xbins = 30, shape = 1, xbnds = range(x), ybnds = range(y))
{
    call<-match.call()
    n <- length(x)
    if(length(y) != n)
        stop("Vector length mismatch")
    if(diff(xbnds) == 0)
        stop("x range must be positive")
    if(diff(ybnds) == 0)
        stop("y range must be positive")
    jmax <- floor(xbins + 1.5001)
    imax <- 2 * floor((xbins * shape)/sqrt(3) + 1.5001)
    dim <- c(imax, jmax)
    lmax <- jmax * imax
    ans <- .Fortran("hbin",
                    x = as.double(x),
                    y = as.double(y),
                    cell= integer(lmax),
                    cnt = integer(lmax),
                    xcm = double(lmax),
                    ycm = double(lmax),
                    xbins = as.double(xbins),
                    shape = as.double(shape),
                    xbnds = as.double(xbnds),
                    ybnds = as.double(ybnds),
                    dim = as.integer(dim),
                    n = as.integer(n),
                    PACKAGE = "hexbin")[-(1:2)]
    ## delele data and extraneous stuff
    nc <- ans$n
    length(ans$cell) <- nc
    length(ans$cnt) <- nc
    length(ans$xcm) <- nc
    length(ans$ycm) <- nc
    if(sum(ans$cnt) != n) warning("Lost counts in binning")
    ans$call <- call
    class(ans) <- "hexbin"
    ans
}## hexbin

hboxplot <- function(bin1, xbnds = bin1$xbnds, ybnds = bin1$ybnds,
                     density = c(-1,-1), border = c(FALSE, FALSE),
                     pen = c(2, 3), unzoom = 1.04, reshape = FALSE,
                     xlab = "", ylab = "", ...)
{
    ## Arguments:
    ##	bin1 = hexagon bin object
    ## 	density=    for hexagon graph paper
    ## 	border      plot the border of the hexagon, use T for
    ##                   hexagon graph paper

### FIXME: This is (almost?) identical to first part of hdiffplot()

    ##_______________ Collect computing constants______________
    tmp1 <- hcell2xy(bin1)
    fix <- missing(xbnds)
    if(fix) {
        xbnds <- bin1$xbnds
        ybnds <- bin1$ybnds
        nxbnds <- range(tmp1$x)
        nybnds <- range(tmp1$y)
    }
    else { ## !fix
        if(missing(ybnds)) ## FIXME: this just return()ed
            ## MM thinks even warning() is too much -- rather check consistency
            warning("`xbnds' specified;  `ybnds' not...")
        nxbnds <- xbnds
        nybnds <- ybnds
    }
    if(fix || reshape) {
        shape <- bin1$shape

        opar <- par(c("pin","mai"))
        on.exit(par(opar))
        pin <- opar$pin
        mai <- opar$mai
        xsize <- pin[1]
        ysize <- pin[2]
        if(shape * xsize <= ysize) {
            inc <- (ysize - shape * xsize)/2
            mai[1] <- mai[1] + inc
            mai[3] <- mai[3] + inc
            ysize <- shape * xsize
        }
        else {
            inc <- (xsize - ysize/shape)/2
            mai[2] <- mai[2] + inc
            mai[4] <- mai[4] + inc
            xsize <- ysize/shape
        }
        par(mai = mai)
    }
    ##___zoom in scaling with expanding to avoid hexagons outside plot frame___
    if(fix) {
        ratiox <- diff(nxbnds)/diff(xbnds)
        ratioy <- diff(nybnds)/diff(ybnds)
        ratio <- max(ratioy, ratiox)
        nxbnds <- mean(nxbnds) + c(-1,1)*(unzoom * ratio * diff(xbnds))/2
        nybnds <- mean(nybnds) + c(-1,1)*(unzoom * ratio * diff(ybnds))/2
    }
    #browser()
    plot(nxbnds, nybnds, type = "n", xlab = xlab, ylab = ylab, ...)
    cnt <- bin1$cnt
    xbins <- bin1$xbins
    shape <- bin1$shape
    xnew <- tmp1$x
    ynew <- tmp1$y
    ##__________________ Construct a hexagon___________________
    dx <- (0.5 * diff(bin1$xbnds))/xbins
    dy <- (0.5 * diff(bin1$ybnds))/(xbins * shape * sqrt(3))
    polyx <- c(dx, dx,  0,    -dx, -dx,   0,    NA)
    polyy <- c(dy,-dy, -2*dy, -dy,  dy, 2 * dy, NA)
    ##_______________ Full Cell Plotting_____________________
    n <- length(xnew)
    seven <- rep.int(7, n)
    pltx <- rep.int(polyx, n) + rep.int(xnew, seven)
    plty <- rep.int(polyy, n) + rep.int(ynew, seven)
    polygon(pltx, plty, density= density[2], border = border[2], col= pen[2])

    ##______________Plotting medians___________________________
    if(is.null(bin1$erode)) {
        warning("No erode component")
    }
    else {
        med <- bin1$erode == max(bin1$erode)
        xnew <- xnew[med]
        ynew <- ynew[med]
        n <- length(xnew)
        seven <- rep.int(7, n)
        pltx <- rep.int(polyx, n) + rep.int(xnew, seven)
        plty <- rep.int(polyy, n) + rep.int(ynew, seven)
        polygon(pltx, plty, density= density[1], border= border[1], col= pen[1])
    }
    ##_____________reset graphics_________________________________
    if(fix || reshape) {
        on.exit()
        invisible( par(opar) )
    }
    else "done"
}# hboxplot()

hcell <- function(x, y, xbins = 30, shape = 1,
                  xbnds = range(x), ybnds = range(y),
                  verbose = getOption("verbose"))
{
    n <- length(x)
    if(length(y) != n)
        stop("Vector length mismatch")
    if(diff(xbnds) == 0)
        stop("No x range")
    if(diff(ybnds) == 0)
        stop("No y range")
    jmax <- floor(xbins + 1.5001)
    imax <- 2 * floor((xbins * shape)/sqrt(3) + 1.5001)
    dim <- c(imax, jmax)
    if(verbose) cat("OK before calling .Fortran(\"hcell\",*)\n")
    .Fortran("hcell",
             x = as.double(x),
             y = as.double(y),
             cell = integer(n),
             n = as.integer(n),
             xbins = as.double(xbins),
             shape = as.double(shape),
             xbnds = as.double(xbnds),
             ybnds = as.double(ybnds),
             dim = as.integer(dim),
             PACKAGE = "hexbin")[
             c("cell","n","xbins", "shape", "xbnds", "ybnds")]
}

hcell2xy <- function(bin)
{
    xbins <- bin$xbins
    xbnds <- bin$xbnds
    c3 <- diff(xbnds)/xbins
    ybnds <- bin$ybnds
    c4 <- (diff(ybnds) * sqrt(3))/(2 * bin$shape * xbins)
    jmax <- bin$dim[2]
    cell <- bin$cell - 1
    i <- cell %/% jmax
    j <- cell %% jmax
    y <- c4 * i + ybnds[1]
    x <- c3 * ifelse(i %% 2 == 0, j, j + 0.5) + xbnds[1]
    list(x = x, y = y)
}

### FIXME : Use S4 methods {i.e. library(methods) } to define
### -----   an `hexagon bin' class -- use at least is(*,) on these

hdiffplot <-
    function(bin1, bin2, xbnds, ybnds,
             density = rep.int(-1,6), border = rep.int(FALSE, 6), pen = 2:7,
             size = 0.1, lwd = 2, eps = 1e-6, unzoom = 1.04,
             xlab = "", ylab = "", main = deparse(mycall), ...)
{
    ## Arguments:
    ##	bin1	: hexagon bin object
    ##  bin2	: hexagon bin object
    ##            bin objects must have the same plotting bounds and shape
    ## 	density : for hexagon graph paper
    ## 	border  : plot the border of the hexagon, use TRUE for
    ##            hexagon graph paper

    ##_______________ Collect computing constants______________
    mycall <- sys.call()
    tmp1 <- hcell2xy(bin1)
    tmp2 <- hcell2xy(bin2)
    fix <- missing(xbnds)
    if(fix) {
        xbnds <- bin1$xbnds
        ybnds <- bin1$ybnds
        nxbnds <- range(tmp1$x, tmp2$x)
        nybnds <- range(tmp1$y, tmp2$y)
        shape <- bin1$shape
        opar <- par(c("pin","mai"))
        on.exit(par(opar))
        pin <- opar$pin
        mai <- opar$mai
        xsize <- pin[1]
        ysize <- pin[2]
        if(shape * xsize <= ysize) {
            inc <- (ysize - shape * xsize)/2
            mai[1] <- mai[1] + inc
            mai[3] <- mai[3] + inc
            ysize <- shape * xsize
        }
        else {
            inc <- (xsize - ysize/shape)/2
            mai[2] <- mai[2] + inc
            mai[4] <- mai[4] + inc
            xsize <- ysize/shape
        }
        par(mai = mai)
        ##_zoom in scaling with expanding to avoid hexagons outside plot frame__
        ratiox <- diff(nxbnds)/diff(xbnds)
        ratioy <- diff(nybnds)/diff(ybnds)
        ratio <- max(ratioy, ratiox)

        nxbnds <- mean(nxbnds) + c(-1,1)*(unzoom * ratio * diff(xbnds))/2
        nybnds <- mean(nybnds) + c(-1,1)*(unzoom * ratio * diff(ybnds))/2

    }
    else { ## !fix
        if(missing(ybnds)) ## FIXME: this just return()ed
            ## MM thinks even warning() is too much -- rather check consistency
            warning("`xbnds' specified;  `ybnds' not...")
        nxbnds <- xbnds
        nybnds <- ybnds
    }
    if(missing(main) && length(mycall) >= 4) {## improve default 'main'
        mycall[4] <- as.call(quote(.....()))
        if(length(mycall) > 4) mycall <- mycall[1:4]
    }

    plot(nxbnds, nybnds, type = "n", main = main, xlab = xlab, ylab = ylab, ...)
    ##__________________ Construct hexagon___________________
    xbins <- bin1$xbins
    shape <- bin1$shape
    dx <- (0.5 * diff(bin1$xbnds))/xbins
    dy <- (0.5 * diff(bin1$ybnds))/(xbins * shape * sqrt(3))
    polyx <- c(dx, dx,   0,  -dx,-dx,  0,  NA)
    polyy <- c(dy,-dy, -2*dy,-dy, dy, 2*dy,NA)
    ##_______________ Full Cell Plotting for Unique Bin1 Cells_________________
    sub1 <- match(bin1$cell, bin2$cell)
    na1 <- is.na(sub1)
    if(any(na1)) {
        xnew <- tmp1$x[na1]
        ynew <- tmp1$y[na1]
        n <- length(xnew)
        seven <- rep.int(7, n)
        pltx <- rep.int(polyx, n) + rep.int(xnew, seven)
        plty <- rep.int(polyy, n) + rep.int(ynew, seven)
        polygon(pltx, plty, density = density[1], border = border[1], col = pen[1])
    }
    ##_______________ Full Cell Plotting for Common Cells_____________________
    if(any(!na1)) {
        xnew <- tmp1$x[!na1]
        ynew <- tmp1$y[!na1]
        n <- length(xnew)
        seven <- rep.int(7, n)
        pltx <- rep.int(polyx, n) + rep.int(xnew, seven)
        plty <- rep.int(polyy, n) + rep.int(ynew, seven)
        polygon(pltx, plty, density = density[2], border = border[2], col = pen[2])
    }
    ##_______________ Full Cell Plotting for Unique Bin2 Cells_________________
    sub2 <- match(bin2$cell, bin1$cell)
    na2 <- is.na(sub2)
    if(any(na2)) {
        xnew <- tmp2$x[na2]
        ynew <- tmp2$y[na2]
        n <- length(xnew)
        seven <- rep.int(7, n)
        pltx <- rep.int(polyx, n) + rep.int(xnew, seven)
        plty <- rep.int(polyy, n) + rep.int(ynew, seven)
        polygon(pltx, plty, density = density[3], border = border[3], col = pen[3])
    }
    ##_____________________________Plot Median Cells___________________________
    if(!is.null(bin1$erode) && !is.null(bin2$erode)) {
        med1 <- bin1$erode == max(bin1$erode)
        xold <- tmp1$x[med1]
        yold <- tmp1$y[med1]
        xold <- xold[1]
        yold <- yold[1]
        med2 <- bin2$erode == max(bin2$erode)
        xnew <- tmp2$x[med2]
        ynew <- tmp2$y[med2]
        xnew <- xnew[1]
        ynew <- ynew[1]
        if(abs(xnew - xold) + abs(ynew - yold) > eps) {
            pltx <- polyx + rep.int(xold, 7)
            plty <- polyy + rep.int(yold, 7)
            polygon(pltx, plty,  density = density[4], border = border[4], col = pen[4])
            pltx <- polyx + rep.int(xnew, 7)
            plty <- polyy + rep.int(ynew, 7)
            polygon(pltx, plty, density = density[5], border = border[5], col = pen[5])
            arrows(xold, yold, xnew, ynew, length = size, lwd = lwd)
        }
        else {
            pltx <- polyx + rep.int(xold, 7)
            plty <- polyy + rep.int(yold, 7)
            polygon(pltx, plty, density = density[6], border = border[6], col = pen[6])
        }
    }
    ##________________Clean Up_______________________________________________
    if(fix) {
        on.exit()
        invisible( par(opar) )
    } else "Done"
} ## hdiffplot()

erode.hexbin <- function(bin, cdfcut = 0.5)
{
    if(!inherits(bin,"hexbin")) stop("first argument must be a hexbin object")
    bin.att<-attributes(bin)
    cell <- bin$cell
    cnt <- bin$cnt
    val <- rep.int(0, length(cell))
    tmp <- sort(cnt)
    cdf <- cumsum(tmp)/sum(cnt)
    good <- cdfcut <= cdf
    if(!any(good))
        return("no cells selected")
    crit <- min(tmp[good])
    good <- crit <= cnt
    cell <- cell[good]
    cnt <- cnt[good]
    bin$cell <- cell
    bin$cnt <- cnt
    n <- length(cell)
    bdim <- bin$dim
    L <- bdim[1] * bdim[2]

    ans <- .Fortran("herode",
                    cell  = as.integer(cell),
                    cnt   = as.integer(cnt),
                    n     = n,
                    bdim  = as.integer(bdim),
                    erode = integer(L),
                    ncnt  = integer(L),
                    ncell = integer(L),
                    sides = integer(L),
                    neib  = integer(6 * L),
                    exist = logical(L + 1),
                    PACKAGE = "hexbin") $ erode
    length(ans) <- n
    bin$erode <- ans
    class(bin) <- c(class(bin),"erodebin")
    bin
}

#hexagons.erode <- function(bin1, xbnds = bin1$xbnds, ybnds = bin1$ybnds,
#                           density, border = c(FALSE, FALSE),
#                           pen = c(2, 3), reshape = FALSE){
#
#}


hexagons <-
function(dat, style = c("colorscale", "centroids", "lattice",
              "nested.lattice", "nested.centroids"),
         minarea = 0.05, maxarea = 0.8,
         mincnt = 1, maxcnt = max(dat$cnt), trans = NULL,
         colorcut = seq(1, 0, length = 17),
         density = NULL, border = NULL, pen = NULL,
         colramp = function(n){ LinGray(n,beg = 90, end = 15) },
         verbose = getOption("verbose"))
{
    ## Warning:  presumes the plot has the right shape and scales
    ##           See plot.hexbin()
    ## Arguments:
    ##	dat= hexagon bin object
    ##  style = type of plotting
    ##          'centroids' =  symbol area is a function of the count,
    ##                  approximate location near cell center of
    ##                  mass without overplotting
    ##          'lattice' = symbol area is a function of the count,
    ##                  plot at lattice points
    ##          'colorscale'   = gray scale plot,
    ##                  color number determined by
    ##                  transformation and colorcut,
    ##                  area = full hexagons.
    ##          'nested.lattice'= plots two hexagons
    ##		   background hexagon
    ##			 area=full size
    ##			 color number by count in
    ##                        powers of 10 starting at pen 2
    ##                  foreground hexagon
    ##		   	 area by log10(cnt)-floor(log10(cnt))
    ##                        color number by count in
    ##                         powers of 10 starting at pen 12
    ##          'nested.centroids' = like nested.lattice but counts <10 are plotted
    ## 	minarea =  minimum symbol area as fraction of the binning cell
    ##  maxarea =  maximum symbol area as fraction of the binning cell
    ##  mincnt  =  minimum count accepted in plot
    ##  maxcnt  =  maximum count accepted in plot
    ##	trans   =  a transformation scaling counts into [0,1] to be applied
    ##		   to the counts for options 'centroids','lattice','colorscale':
    ##                   default=(cnt-mincnt)/(maxcnt-mincnt)
    ##  colorcut=  breaks for translating values between 0 and 1 into
    ##		   color classes.  Default= seq(0,1,17),
    ## 	density =  for hexagon graph paper
    ## 	border     plot the border of the hexagon, use TRUE for
    ##             hexagon graph paper
    ## Symbol size encoding:
    ##   Area= minarea + scaled.count*(maxarea-minarea)
    ##   When maxarea==1 and scaled.count==1, the hexagon cell
    ##   is completely filled.
    ##
    ##   If small hexagons are hard to see increase minarea.
    ## For gray scale encoding
    ##    Uses the counts scaled into [0,1]
    ##    Default gray cutpoints seq(0,1,17) yields 16 color classes
    ##    The color number for the first class starts at 2.
    ##         motif coding: black 15 white puts the first of the
    ##                       color class above the background black
    ##    The function subtracts 1.e-6 from the lower cutpoint to include
    ##    the boundary
    ## For nested scaling see the code
    ## Count scaling alternatives
    ##
    ##  log 10 and Poisson transformations
    ## 	trans <- function(cnt) log10(cnt)
    ##     min  inv   <- function(y) 10^y
    ##
    ##       trans <- function(cnt) sqrt(4*cnt+2)
    ##       inv   <- function(y) (y^2-2)/4
    ## Perceptual considerations.
    ##    Visual response to relative symbol area is not linear and varies from
    ##    person to person.  A fractional power transformation
    ##    to make the interpretation nearly linear for more people
    ##    might be considered.  With areas bounded between minarea
    ##    and 1 the situation is complicated.
    ##
    ##    The local background influences color interpretation.
    ##    Having defined color breaks to focus attention on
    ##    specific countours can help.
    ##
    ## Plotting the symbols near the center of mass is not only more accurate,
    ##    it helps to reduce the visual dominance of the lattice structure.  Of
    ##    course higher resolution binning reduces the possible distance between
    ##    the center of mass for a bin and the bin center.  When symbols
    ##    nearly fill their bin, the plot appears to vibrate.  This can be
    ##    partially controlled by reducing maxarea or by reducing
    ##    contrast.


    ##____________________Initial checks_______________________
    style <- match.arg(style) # so user can abbreviate
    #print(style)
    if(minarea <= 0)
        stop("hexagons cannot have a zero area, change minarea")
    if(maxarea > 1) warning("maxarea>1, hexagons may overplot")
    ##_______________ Collect computing constants______________
    cnt <- dat$cnt
    xbins <- dat$xbins
    shape <- dat$shape
    tmp <- hcell2xy(dat)
    good <- mincnt <= cnt & cnt <= maxcnt
    xnew <- tmp$x[good]
    ynew <- tmp$y[good]
    cnt <- cnt[good]
    sx <- xbins/diff(dat$xbnds)
    sy <- (xbins * shape)/diff(dat$ybnds)
    ##___________Transform Counts to Radius_____________________
    switch(style,
           "centroids" = ,
           "lattice" = ,
           "colorscale" = {
               if(is.null(trans)) {
                   rcnt <- {
                       if(maxcnt == mincnt) rep.int(1, length(cnt))
                       else (cnt - mincnt)/(maxcnt - mincnt)
                   }
               }
               else {
                   rcnt <- (trans(cnt) - trans(mincnt)) /
                       (trans(maxcnt) - trans(mincnt))
                   if(any(is.na(rcnt)))
                       stop("bad count transformation")
               }
               area <- minarea + rcnt * (maxarea - minarea)
               area <- pmin(area, maxarea)
               radius <- sqrt(area)
           },
           nested.lattice = ,
           nested.centroids = {
               diffarea <- maxarea - minarea
               step <- 10^floor(log10(cnt))
               f <- (cnt/step - 1)/9
               area <- minarea + f * diffarea
               area <- pmax(area, minarea)
               area <- pmin(area, maxarea)
               radius <- sqrt(area)
           }
           )
    ##______________Set Colors_____________________________
    switch(style,
           centroids = ,
           lattice = {if(is.null(pen))pen <- rep.int(1, length(cnt))
                      else if(length(pen)==length(cnt)) break
                      else if(length(pen)==1)pen <- rep.int(pen,length(cnt))
                      else stop("Pen is wrong length")
                      },
           nested.lattice = ,
           nested.centroids = {
               if(length(dim(pen))==2 & !is.null(pen)){
                 if( (dim(pen)[1] != length(cnt))
                    & ( dim(pen)[1] != ceiling(log10(max(cnt))) ) )
                   {
                   #print(dim(pen)[1])
                   #print(ceiling(log10(max(cnt))))
                   stop ("pen is not of right dimension")}
                 if( dim(pen)[1] == ceiling(log10(max(cnt))) ){
                   #tmp<-pen
                   #browser()
                   ind<-ceiling(log10(bin$cnt))
                   ind[ind==0]<-1
                   pen<-pen[ind,]
                   #print(pen)
                 }
                 else break
               }
               else{
                 pen<-matrix(0,length(cnt),2)
                 pen[,1] <- floor(log10(cnt)) + 2
                 pen[,2] <- pen[,1]+10
               }
           },
           colorscale = {
               ## MM: Following is quite different from bin2d's
               if(colorcut[1] > colorcut[length(colorcut)]){
                   colorcut[1] <- colorcut[1] + 1e-06
                   colorcut[length(colorcut)] <-
                       colorcut[length(colorcut)] - 1e-06
               } else {
                   colorcut[1] <- colorcut[1] - 1e-06
                   colorcut[length(colorcut)] <-
                       colorcut[length(colorcut)] + 1e-06

               }
               colgrp <- cut(rcnt, colorcut,labels = FALSE)
               ##NL: colramp must be a function accepting an integer n
               ##    and returning n colors
               clrs <- colramp(max(colgrp))
               pen <- clrs[colgrp]
           })
    ##__________________ Construct a hexagon___________________
    ## The inner and outer radius for hexagon in the scaled plot
    inner <- 0.5
    outer <- (2 * inner)/sqrt(3)
    ## Now construct a point up hexagon symbol in data units
    dx <- inner/sx
    dy <- outer/(2 * sy)
    rad <- sqrt(dx^2 + dy^2)
    polyx <- c(dx, dx, 0,  - dx,  - dx, 0, NA)
    polyy <- c(dy,  - dy, -2 * dy,  - dy, dy, 2 * dy, NA)
    ##_______________ Full Cell  Plotting_____________________
    switch(style,
           colorscale = {
               n <- length(xnew)
               seven <- rep.int(7, n)
               pltx <- rep.int(polyx, n) + rep.int(xnew, seven)
               plty <- rep.int(polyy, n) + rep.int(ynew, seven)
               polygon(pltx, plty, density = density,
                       border = if(!is.null(border)) border else pen, col = pen)
             },
           nested.lattice = ,
           nested.centroids = {
               n <- length(xnew)
               seven <- rep.int(7, n)
               pltx <- rep.int(polyx, n) + rep.int(xnew, seven)
               plty <- rep.int(polyy, n) + rep.int(ynew, seven)
               polygon(pltx, plty, density = density,
                       border = if(!is.null(border)) border else pen[,1],
                       col = pen[,1])
           }
           )
    if(style == "colorscale") return("done `colorscale'")
    ##__________________ Symbol Center adjustments_______________
    if(style == "centroids" || style == "nested.centroids") {
        xcm <- dat$xcm[good]
        ycm <- dat$ycm[good]
        ## Store 12 angles around a circle and the replicate the first
        ## The actual length for these vectors is determined by using
        ## factor use below
        k <- sqrt(3)/2
        cosx <- c(1, k, 0.5, 0, -0.5,  - k, -1,  - k, -0.5, 0, 0.5, k, 1)/sx
        siny <- c(0, 0.5, k, 1, k, 0.5, 0, -0.5,  - k, -1,  - k, -0.5, 0)/sy
        ## Compute distances for differences after scaling into
        ## [0,size] x [0,aspect*size]
        ## Then there are size hexagons on the x axis
        dx <- sx * (xcm - xnew)
        dy <- sy * (ycm - ynew)
        dlen <- sqrt(dx^2 + dy^2)
        ## Find the closest approximating direction of the 12 vectors above
        cost <- ifelse(dlen > 0, dx/dlen, 0)
        tk <- (6 * acos(cost))/pi
        tk <- round(ifelse(dy < 0, 12 - tk, tk)) + 1
	## Select the available length for the approximating vector
        hrad <- ifelse(tk %% 2 == 1, inner, outer)
	## Rad is either an inner or outer approximating radius.
        ## If dlen + hrad*radius <= hrad, move the center dlen units.
        ## Else move as much of dlen as possible without overplotting.
        fr <- pmin(hrad * (1 - radius), dlen) # Compute the symbol centers
        ## fr is the distance for the plot [0,xbins]x[0,aspect*xbins]

        ## cosx and siny give the x and y components of this distance
        ## in data units
        xnew <- xnew + fr * cosx[tk]
        ynew <- ynew + fr * siny[tk]
    }
    ## ________________Sized Hexagon Plotting__________________
    ## scale the symbol by radius and add to the new center
    n <- length(radius)
    if(verbose)
        cat('length = ',length(pen),"\n", 'pen = ', pen+1,"\n")
    #switch(style,
    #       centroids = ,
    #       lattice = {if(is.null(pen))pen <- rep.int(1, n)
    #                   else pen <- rep.int(pen, n)},
    #      nested.lattice = ,
    #       nested.centroids ={
    #         if(
    #            pen[,2] <- pen[,1] + 10
    #     }           )
    seven <- rep.int(7, n)
    pltx <- rep.int(polyx, n) * rep.int(radius, seven) + rep.int(xnew, seven)
    plty <- rep.int(polyy, n) * rep.int(radius, seven) + rep.int(ynew, seven)
    switch(style,
           "centroids" = ,
           "lattice" = {
               polygon(pltx, plty,
                       border = if(!is.null(border)) border else pen,
                       col = pen)
           },
           "nested.lattice" = ,
           "nested.centroids" = {
               polygon(pltx, plty,
                       border = if(!is.null(border)) border else pen[,2],
                       col = pen[,2])
           })

} ## hexagons()


## Add a legend to a plot

hex.legend <-
function(legend, ysize, lcex, inner,
	 style = c("colorscale", "centroids", "lattice",
	 "nested.lattice", "nested.centroids"),
	 minarea = 0.05, maxarea = 0.8, mincnt = 1, maxcnt, trans = NULL,
	 inv = NULL, colorcut, density = NULL, border = NULL, pen = NULL,
	 colramp = function(n) { LinGray(n,beg = 90,end = 15) })
{
    style <- match.arg(style) # check; user can abbreviate

    if (style %in% c("centroids", "lattice", "colorscale")) {
	## _______________tranformations_______________________
	if(is.null(trans)) {
	    sc <- maxcnt - mincnt
	    bnds <- round(mincnt + sc * colorcut)
	}
	else {
	    if(!is.function(trans) && !is.function(inv))
		stop("'trans' and 'inv' must both be functions if 'trans' is not NULL")
	    con <- trans(mincnt)
	    sc <- trans(maxcnt) - con
	    bnds <- round(inv(con + sc * colorcut))
	}
    }

    ## _______________Plotting______________________________
    switch(style,
	   "colorscale" = {
	       n <- length(bnds)
	       spacing <- ysize/(n + 3)
	       inner <- min(legend/3.5, (sqrt(3) * spacing)/2)
	       dx <- inner/2
	       dy <- dx/sqrt(3)
	       midx <- legend/3
	       textx <- (2 * legend)/3
	       tx <- c(dx, dx, 0,  - dx,  - dx, 0, NA) + midx
	       ty <- c(dy,  - dy, -2 * dy,  - dy, dy, 2 *
		       dy,NA)
	       pen <- colramp(n)
	       for(i in 1:n) {
		   polygon(tx, ty + i * spacing,
			   col = pen[i], border = border,
			   density = density)# or pen[i], opt.? MM
		   text(textx, (i - 0.5) * spacing, as.character(bnds[i]))
	       }
	       text(textx, (n - 0.5) * spacing, as.character(bnds[n]))
	       text(legend/2, (n + 1.5) * spacing, "Counts", cex = 1.7 * lcex)
	   },

	   "centroids" = ,
	   "lattice" = {
	       ## NL Solved hex overlap problem on legend
	       ## Need to tackle too many categories
	       dx <- inner/2
	       dy <- dx/sqrt(3)
	       radius <- sqrt(minarea + (maxarea - minarea) * colorcut)
	       n <- length(radius)
	       xmid <- legend/3
	       tx <- rep.int(c(dx,	dx,	0, -dx, -dx, 0,	   NA), n)
	       ty <- rep.int(c(dy, -dy, -2*dy, -dy,	 dy, 2*dy, NA), n)
	       rad.units <- 2*dy*radius
	       shift <- c(rad.units[1],rad.units[1:(n-1)]+rad.units[2:n])
	       labht <- max(strheight(as.character(bnds),cex = lcex))
	       shift <- ifelse(labht > shift,labht,shift)
	       seven <- rep.int(7, n)
	       inc <- ysize/(n+3)
	       if(inc > max(shift)) y <- inc * 1:n
	       else {
		 y <- cumsum(shift)
		 extra.slop <- (n * inc) - y[n]
		 shift <- c(shift[1],shift[-1]+ (extra.slop/(n-1)))
		 y <- cumsum(shift)  ## (y+(1/n)*extra.slop)-y[1]
		 ## delta <- max(log(shift))-min(log(shift))
		 ## fudge <- extra.slop*(diff(log(shift))/delta)
		 ## y<- c(y[1], y[-1]+ fudge )
	       }
	       textx <- rep.int((2 * legend)/3, n)
	       ##  ____________________plotting______________________
	       if(is.null(pen)) pen <- 1
	       if(is.null(border)) border <- pen
	       pltx <- tx * rep.int(radius, seven) + rep.int(xmid, 7 * n)
	       plty <- ty * rep.int(radius, seven) + rep.int(y, seven)
	       polygon(pltx, plty, col = pen, border= border, density= density)
	       text(textx, y, as.character(bnds), cex = lcex)
	       text(legend/2, (n + 2) * inc, "Counts", cex = 1.7 * lcex)
	   },

	   "nested.lattice" = ,
	   "nested.centroids" = {
	       ## _____________x scaling_____________________________
	       numb <- cut(floor(legend/inner), breaks = c(-1, 0, 2,4))
	       ## Note: In old code
	       ##	top breaks=c(-1,0,2,4,8), numb<- 5 and size=1:9
	       if(is.na(numb))
		   numb <- 4
	       switch(numb,
		      {warning("not enough space for legend"); return()},
		      size <- 5,
		      size <- c(1, 5, 9),
		      size <- c(1, 3, 5, 7, 9))
	       xmax <- length(size)
	       diffarea <- maxarea - minarea
	       radius <- sqrt(minarea + (diffarea * (size - 1))/9)
	       txt <- as.character(size)
	       ##___________________y scaling_____________________
	       lab <- c("Ones", "Tens", "Hundreds",
			"Thousands", "10 Thousands", "100 Thousands",
			"Millions", "10 Millions",
			"100 Millions", "Billions")
	       power <- floor(log10(maxcnt)) + 1
	       dx <- inner/2
	       dy <- dx/sqrt(3)
	       if(ysize/power < 16 * dy)
		   {warning("Not enough height for legend"); return()}
	       xmid <- legend/2
	       x <- inner * (1:xmax - (1 + xmax)/2) + xmid
	       n <- length(x)
	       tx <- rep.int(c(dx, dx,  0,	 -dx,-dx,  0,	NA), n)
	       ty <- rep.int(c(dy,-dy, -2*dy,-dy, dy, 2*dy, NA), n)
	       seven <- rep.int(7, n)
	       yinc <- 16 * dy
	       y <- rep.int(3 * dy - yinc, xmax)
	       ##  ____________________plotting______________________
	       if(is.null(pen)) {
		 pen <- matrix(1:power,power,2)
		 pen[,1] <- pen[,1]+1
		 pen[,2] <- pen[,2]+11
	       }
	       if(is.null(border)) border <- FALSE
	       for(i in 1:power) {
		   y <- y + yinc
		   polygon(x= tx + rep.int(x, seven),
			   y= ty + rep.int(y, seven),
			   col = pen[i,1], border = border)
		   polygon(x= tx * rep.int(radius, seven) + rep.int(x, seven),
			   y= ty * rep.int(radius, seven) + rep.int(y, seven),
			   col = pen[i,2], border = border)
		   text(x,     y   - 4.5 * dy, txt,    adj = 0.5)
		   text(xmid, y[1] + 4.5 * dy, lab[i], adj = 0.5)
	       }
	   })## switch(style = "nested.*" ..)
}

hmatplot <- function(namemat, rlabels, clabels,
                     brd = c(0.5, 0.7, 0.7, 0.5),
                     mai = rep.int(0.4, 4), unzoom = 1.04, cex = 1.5,
                     border = list(hbox=c(FALSE, FALSE),
                                   hdiff=rep.int(FALSE,6)),
                     pen = list( hbox = c(2, 3),hdiff = 4:9),
                     density = list(hbox = c(-1,-1),
                                    hdiff= c(-1, -1, -1, 0, 0, 0)),
                     size = 0.06, lwd = 2)
{
    ##___Get initial constants and check arguments_______________________
    nr <- nrow(namemat)
    nc <- ncol(namemat)
    if(missing(rlabels))
        rlabels <- rep.int("", nr)
    if(missing(clabels))
        clabels <- rep.int("", nc)
    if(any(mai > brd))
        stop("Border `brd' must be larger than margin `mai' on every side")

    ##frame() ## <<- MM: always?
    plot.new()

    ##_____________________________________________________________
    ##
    ##  Step 1 - Find x and y ranges that maximize the resolution
    ##           and maintain the shape
    tnam <- as.vector(namemat)
    bin1 <- get(tnam[1])
    tmp <- hcell2xy(bin1)
    rx <- range(tmp$x)
    ry <- range(tmp$y)
    for(i in 2:length(tnam)) {
        tmp <- hcell2xy(get(tnam[i]))
        rx <- range(rx, tmp$x)
        ry <- range(ry, tmp$y)
    }
    xbnds <- bin1$xbnds
    ybnds <- bin1$ybnds
    ratiox <- diff(rx)/diff(xbnds)
    ratioy <- diff(ry)/diff(ybnds)
    ratio <- max(ratioy, ratiox)

    rx <- mean(rx) + c(-1,1)*(unzoom * ratio * diff(xbnds))/2
    ry <- mean(ry) + c(-1,1)*(unzoom * ratio * diff(ybnds))/2
    print(rx)
    print(ry)
    ##_________________________________________________________
    ##  Step 2.  set up a matrix of plots
    oldpar <- par(no.readonly=TRUE)
    on.exit(par(oldpar))
    opar <-list()
    opar$global <- par(xpd = TRUE, xaxt = "n", yaxt = "n")
    din <- par("din")
    xsize <- din[1] - brd[2] - brd[4]
    ysize <- din[2] - brd[1] - brd[3]
    nrplot <- 2 * nr - 1
    ncplot <- 2 * nc - 1
    nshape <- (ysize/nrplot)/(xsize/ncplot)
    shape <- bin1$shape
    if(nshape < shape) {
        inc <- (xsize * (1 - nshape/shape))/2
        brd <- brd + c(0, inc, 0, inc)
        xsize <- xsize - 2 * inc
    }
    else {
        inc <- (ysize * (1 - shape/nshape))/2
        brd <- brd + c(inc, 0, inc, 0)
        ysize <- ysize - 2 * inc
    }
    pxsize <- xsize/ncplot
    pysize <- ysize/nrplot
    st <- c((brd[2] - mai[2])/din[1],
            (brd[2] + pxsize + mai[4])/din[1],
            (din[2] - brd[3] - pysize - mai[1])/ din[2],
            (din[2] - brd[3] + mai[3])/din[2])
    pxsize <- pxsize/din[1]
    pysize <- pysize/din[2]
    ##_________________________________________________________
    ## Step 3. produce each boxplot in the structure
    #browser()
    opar$boxpar<-list()
    for(i in seq(length=nr)) {
        for(j in seq(length=nc)) {
            px <- 2 * (j - 1) * pxsize
            py <- 2 * (i - 1) * pysize
            opar$boxpar <- c(opar$boxpar,
                             par(new=TRUE, fig = st + c(px, px, -py, -py),
                                 mai = mai))
            bin <- get(namemat[i, j])
            hboxplot(bin, rx, ry, density=density$hbox, border=border$hbox, pen = pen$hbox)
            if(i == 1)
                mtext(side = 3, clabels[j], line = 1, cex = cex)
            if(j == 1)
                mtext(side = 2, rlabels[i], line = 1, cex = cex)
        }
    }
    ##_____________________________________________________________
    ## Step 4. produce a difference plot for adjacent pairs in each row
    opar$diffpar.row<-list()
    if(nc > 1) {
        for(i in 1:nr) {
            py <- 2 * (i - 1) * pysize
            for(j in 1:(nc - 1)) {
                px <- (2 * j - 1) * pxsize
                opar$diffpar.row <- c(opar$diffpar.row,
                                      par(new=TRUE,
                                          fig = st + c(px, px, -py, -py)))
                bin1 <- get(namemat[i, j])
                bin2 <- get(namemat[i, j + 1])

                hdiffplot(bin1, bin2, rx, ry, density = density$hdiff,
                          border = border$hdiff, pen = pen$hdiff,
                          size = size, lwd = lwd, xaxt = "n", yaxt = "n",
                          main = "")
            }
        }
    }
    ##_____________________________________________________________
    ## Step 5.  plot difference plot for adjacent pairs in each column
    opar$diffpar.col<-list()
    if(nr > 1) {
        for(j in 1:nc) {
            px <- 2 * (j - 1) * pxsize
            for(i in 1:(nr - 1)) {
                py <- (2 * i - 1) * pysize
                of <- par(new=TRUE, fig = st + c(px, px,  - py,  - py))
                if(!any("fig" == names(opar)))
                  opar$diffpar.col <- c(opar$diffpar.col, of)
                bin1 <- get(namemat[i, j])
                bin2 <- get(namemat[i + 1, j])
                hdiffplot(bin1, bin2, rx, ry, density = density$hdiff,
                          border = border$hdiff, pen = pen$hdiff,
                          size = size, lwd = lwd, xaxt = "n", yaxt = "n",
                          main = "")
            }
        }
    }
    ##_______________________________________________________________
    ## Step 6.  cleanup
    invisible(list(xbnds = rx, ybnds = ry, xsize=xsize, ysize=ysize,
                   plot.par = opar))
    #invisible(list(xbnds = xbnds, ybnds = ybnds, plot.par = opar))
}## hmatplot()

plot.hexbin <-
    function(x, style = c("colorscale", "centroids", "lattice",
		"nested.lattice", "nested.centroids"),
	     legend = 1, lcex = 1,
	     minarea = 0.04, maxarea = 0.8, mincnt = 1, maxcnt = max(x$cnt),
	     trans = NULL, inv = NULL,
	     colorcut = seq(0, 1, length = min(17, maxcnt)),
	     border = NULL, density = NULL, pen = NULL,
	     colramp = function(n){ LinGray(n,beg = 90,end = 15) },
	     xlab = "", ylab = "", verbose = getOption("verbose"), ...)
{
    ##NL: Added colramp argument to use fancy ramps
    if(!inherits(x,"hexbin")) stop("first argument must be a hexbin object")
    style <- match.arg(style) # check; user can abbreviate
    plot.new()
    ## MM: wrong in R (?):
    #frame()

    ##NL: Needed this to get the legend to work
    #split.screen(c(1,1))
    ##layout(matrix(c(2,1),1,2),c(5,1))
    ##NL: Layout will probably work better
    if(minarea < 0)
	stop("Minimum area must be non-negative")
    if(maxarea > 1)
	warning("Maximum area should be <= 1 this leads to overlapping hexagons")
    if(minarea > maxarea)
	stop("Minarea must be <- maxarea")
    if(length(colorcut) > 1) {
	if(colorcut[1] != 0)
	    stop("Colorcut lower boundary must be 0")
	if(colorcut[length(colorcut)] != 1)
	    stop("Colorcut upper boundary must be 1")
    }
    else {
	colorcut <- 1
    }

    shape <- x$shape
    oldpar <- par(no.readonly=TRUE)
    on.exit( {
	##RG: pars are funny things
	##oldpar$cin <- NULL; oldpar$din<-NULL;
	##oldpar$cra<-NULL;oldpar$csi<-NULL;oldpar$cxy<-NULL;
	##oldpar$gamma<-NULL;
	##RG
	par(oldpar)
	#par(new = FALSE, mai = oldpar$mai, fig = oldpar$fig)
    } )

    din <- oldpar$din
    pin <- oldpar$pin
    mai <- oldpar$mai
    fig <- oldpar$fig
    xsize <- pin[1]
    ysize <- pin[2]
    if(is.logical(legend)) {
	if(legend)
	    stop("Give the legend width")
	else legend <- 0
    }
    xsize <- xsize - legend
    if(xsize < 1)
	stop("plot width too small")
    if(shape * xsize <= ysize) {
	center <- (ysize - shape * xsize)/2
	mai[1] <- mai[1] + center
	mai[3] <- mai[3] + center
	mai[4] <- mai[4] + legend
	ysize <- shape * xsize
    }
    else {
	center <- (xsize - ysize/shape)/2
	mai[2] <- mai[2] + center
	mai[4] <- mai[4] + center + legend
	xsize <- ysize/shape
    }
    if(legend > 0) {
	if(!is.null(trans) && is.null(inv))
	    stop("Must supply the inverse transformation")

	if(verbose)
	    cat("plot.hexbin( legend > 0):  ... hex.legend()\n")

	#screen(1,new = FALSE)## (MM: not in bin2d)
	mai[2] <- mai[2] + xsize
	mai[4] <- mai[4] - legend
	par(fig = fig, mai = mai, new=TRUE)
	plot.new()
	plot.window(c(0, legend), c(0, ysize))
	     ##type = "n", axes = FALSE, xlab = "", ylab = "", main = "")
	inner <- xsize/x$xbins
	outer <- (inner * sqrt(3))/2
	##switch(style,
	##	 lattice = ,
	##	 centroids = {
	##	     if(length(colorcut) * outer > ysize - 1) {
	##		 warning("Colorcut is being shortened")
	##		 colorcut <- seq(0, 1,
	##				 max(1, floor((ysize - 1)/outer)))
	##	     }
	##	 }
	##	 )
	hex.legend(legend, ysize=ysize, lcex = lcex, inner = inner,
		   style= style, minarea= minarea, maxarea= maxarea,
		   mincnt= mincnt, maxcnt= maxcnt, trans=trans, inv=inv,
		   colorcut = colorcut,
		   density = density, border = border, pen = pen,
		   colramp = colramp)

	mai[2] <- mai[2] - xsize
	mai[4] <- mai[4] + legend
	## MM: bin2d does   par(new = TRUE)

	## }
	## screen(1,new = FALSE)## (MM: not in bin2d)
	## if(legend>0) {

	if(verbose) cat("plot.hexbin(): calling par(new=TRUE,..); plot.new()")

      par(new=TRUE, mai = mai, fig = fig)
      plot.new()
    }

    plot(x$xbnds, x$ybnds, type = "n", xlab = xlab, ylab = ylab, ...)
    hexagons(x, style = style,
	     minarea = minarea, maxarea = maxarea,
	     mincnt = mincnt, maxcnt = maxcnt,
	     trans = trans, colorcut = colorcut, density = density,
	     border = border, pen = pen, colramp = colramp, verbose = verbose)

    invisible(par(no.readonly=TRUE))
} ## plot.hexbin()

smooth.hexbin <- function(bin, wts = c(48, 4, 1))
{
    if(!inherits(bin,"hexbin"))
        stop("first argument must be a hexbin object")
    cell <- bin$cell - 1
    n <- length(cell)
    cnt <- bin$cnt
    xbins <- bin$xbins
    bdim <- bin$dim
    row <- bdim[1]
    col <- bdim[2]
    ncol <- col + 4
    nrow <- row + 4
    nmax <- ncol * nrow
    sm <- rep.int(0, nmax)
    nr <- cell %/% col + 2
    nc <- cell %% col + 3
    pad <- rep.int(0, nmax - n)
    cell <- c(nr * ncol + nc, pad)
    cnt <- c(cnt, pad)
    ans <- .Fortran("hsm",
                    cell = as.integer(cell),
                    cnt  = as.integer(cnt),
                    n    = n,
                    nmax = as.integer(nmax),
                    sm   = as.integer(sm),
                    ncol = as.integer(ncol),
                    wts  = as.integer(wts),
                    PACKAGE = "hexbin")[c("cell","cnt","n")]
    n <- ans$n
    length(ans$cell) <- length(ans$cnt) <- n
    ans$xbins <- xbins + 4
    ans$shape <- bin$shape
    ans$xbnds <- bin$xbnds + c(-2, 2)* diff(bin$xbnds)/xbins
    ans$ybnds <- bin$ybnds + c(-2, 2)* diff(bin$ybnds)/xbins
    ans$dim <- c(nrow, ncol)
    ans$wts <- wts
    class(ans) <- c("sm.hexbin", class(bin))## order reversed, originally
    ans
}


identify.hexbin <- function(x, labels = x$cnt, offset = 0, ...)
{
    if(length(labels) != x$n)
        stop("labels not the same length as number of cells")
    ##NL: Should this be a warning?

    ## -> typically default method:
    identify(hcell2xy(x), labels = labels, offset = offset, ...)
}

summary.hexbin <- function(object, ...) {
  cat(paste("Call:", deparse(object$call), "\n"))
  cat("Hexagon Grid Dimensions: ", object$dim[1], "by", object$dim[2],"\n")
  summary(data.frame(cell = object$cell, count = object$cnt,
                     xcenter = object$xcm, ycenter = object$ycm),
          ...)
  ##-> NextMethod()
}

