\name{hboxplot}
\alias{hboxplot}
\title{2-D Generalization of Boxplot}

\description{
  If \code{bin1} is a \code{\link{hexbin}} object with an \code{erode}
  component.  Then, \code{hboxplot()} plots the high counts cells
  selected by the \code{erode} function.  By default, the high counts
  cells contain 50 percent of the counts so analagous to the
  interquartile range.  The function distinguishes the last cells eroded
  using color.  These cells correspond to one definition of the
  bivariate median.
}

\usage{
hboxplot(bin1, xbnds = bin1$xbnds, ybnds = bin1$ybnds,
         density = c(-1,-1), border = c(FALSE, FALSE), pen = c(2, 3),
         unzoom = 1.04, reshape = FALSE,
         xlab = "", ylab = "", \dots)
}

\arguments{
  \item{bin1}{an object of class \code{\link{hexbin}}.}
  \item{xbnds}{global x-axis plotting limits for multiple plots.}
  \item{ybnds}{global y-axis plotting limits for multiple plots. }
  \item{density, border}{arguments for \code{\link{polygon}()} each of
    length two, the first for the median, the second for the other cells.}
  \item{pen}{colors (\dQuote{pen numbers}) for \code{polygon()}.}
  \item{unzoom}{plot limit expansion factor when \code{xbnds} is missing.}
  \item{reshape}{logical value to reshape the plot although \code{xbnds}
    and \code{ybnds} are present.}
  \item{xlab, ylab}{x- and y- axis labels passed to \code{\link{plot}()}.}
  \item{\dots}{additional arguments passed to \code{\link{plot}()}.}
}

\value{% MM: this seems hackish:
 If (missing(xbnds) $|$ reshape)screen.par else 'done'
}

\references{ see in \code{\link{hexagons}}.}

\details{
  The \code{density}, \code{border}, and \code{pen} arguments correspond
  to the \code{\link{polygon}} function calls for plotting two types of
  cells.  The cell types, pen numbers and suggested colors are\cr
  \tabular{lll}{
    TYPE				   \tab PEN \tab COLOR \cr
    cells of bin1                           \tab 2  \tab light gray \cr
    last eroded cells of bin1 (median cells)\tab 1  \tab black \cr
  }

  The erode components of the hexbin objects must be present for the
  medians cells to plot.

  When \code{xbnds} is missing or \code{reshape} is true, the plot
  changes graphics parameters and resets them.   When \code{xbnds} is
  missing the function also zooms in based on the available data to
  provide increased resolution.

  The zoom used the hexagon cell centers.  The unzoom argument backs off
  a bit so the whole hexagon will fit in the plot.

  \code{Hboxplot()} is used as a stand alone function, for producing separate
  legends for \code{\link{hmatplot}()} and for panels in
  \code{\link{hmatplot}()}.
}

\seealso{
  \code{\link{hexbin}}, \code{\link{smooth.hexbin}}, \code{\link{erode.hexbin}},
  \code{\link{hcell2xy}},  \code{\link{hcell}}
  \code{\link{plot.hexbin}}, \code{\link{hmatplot}},
  \code{\link{hexagons}}, \code{\link{hex.legend}}
}

\examples{
##  boxplot of smoothed counts
x <- rnorm(10000)
y <- rnorm(10000)

bin <- hexbin(x,y)
erodebin <- erode.hexbin(smooth.hexbin(bin))

hboxplot(erodebin)
hboxplot(erodebin, density = c(32,7), border = c(2,4))
hboxplot(erodebin, density = c(-1,17),
         main = "hboxplot(erode*(smooth*(.)))")
}
\keyword{hplot}










